import { CommandEntryPoint } from './adapters/codemirror/feature';
export const file_editor_adapters = new Map();
export const notebook_adapters = new Map();
function is_context_menu_over_token(adapter) {
    let position = adapter.get_position_from_context_menu();
    if (!position) {
        return false;
    }
    let token = adapter.virtual_editor.getTokenAt(position);
    return token.string !== '';
}
class LSPCommandManager {
    constructor(app, tracker, suffix) {
        this.app = app;
        this.tracker = tracker;
        this.suffix = suffix;
    }
    add(commands) {
        for (let cmd of commands) {
            this.app.commands.addCommand(this.create_id(cmd), {
                execute: () => this.execute(cmd),
                isEnabled: () => this.is_enabled(cmd),
                isVisible: () => this.is_visible(cmd),
                label: cmd.label
            });
            if (this.should_attach(cmd)) {
                this.attach_command(cmd);
            }
        }
    }
    should_attach(command) {
        if (typeof command.attach_to === 'undefined') {
            return true;
        }
        return command.attach_to.has(this.entry_point);
    }
    create_id(command) {
        return 'lsp:' + command.id + '-' + this.suffix;
    }
}
export class ContextMenuCommandManager extends LSPCommandManager {
    constructor(app, tracker, suffix, rank_group, rank_group_size) {
        super(app, tracker, suffix);
        this.rank_group = rank_group;
        this.rank_group_size = rank_group_size;
    }
    attach_command(command) {
        this.app.contextMenu.addItem({
            selector: this.selector,
            command: this.create_id(command),
            rank: this.get_rank(command)
        });
    }
    add_context_separator(position_in_group) {
        this.app.contextMenu.addItem({
            type: 'separator',
            selector: this.selector,
            rank: this.rank_group + position_in_group
        });
    }
    execute(command) {
        let context = this.current_adapter.get_context_from_context_menu();
        command.execute(context);
    }
    is_enabled() {
        return is_context_menu_over_token(this.current_adapter);
    }
    is_visible(command) {
        let context = this.current_adapter.get_context_from_context_menu();
        return (this.current_adapter && context.connection && command.is_enabled(context));
    }
    get_rank(command) {
        let is_relative = typeof command.is_rank_relative === 'undefined'
            ? true
            : command.is_rank_relative;
        if (is_relative &&
            typeof this.rank_group !== 'undefined' &&
            this.rank_group_size) {
            let relative = typeof command.rank !== 'undefined' ? command.rank : 0;
            return this.rank_group + relative / this.rank_group_size;
        }
        else {
            return typeof command.rank !== 'undefined' ? command.rank : Infinity;
        }
    }
}
export class NotebookCommandManager extends ContextMenuCommandManager {
    constructor() {
        super(...arguments);
        this.selector = '.jp-Notebook .jp-CodeCell .jp-Editor';
        this.entry_point = CommandEntryPoint.CellContextMenu;
    }
    get current_adapter() {
        let notebook = this.tracker.currentWidget;
        return notebook_adapters.get(notebook.id);
    }
}
export class FileEditorCommandManager extends ContextMenuCommandManager {
    constructor() {
        super(...arguments);
        this.selector = '.jp-FileEditor';
        this.entry_point = CommandEntryPoint.FileEditorContextMenu;
    }
    get current_adapter() {
        let fileEditor = this.tracker.currentWidget.content;
        return file_editor_adapters.get(fileEditor.id);
    }
}
//# sourceMappingURL=command_manager.js.map